@@ -23,12 +23,14 @@
 #include "virtobj.h"
 #include "missile.h"
 #include "tank.h"
+#include "files.h"
 
 ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 {
 	gravity = 0.150;
 	viscosity = 0.5;
 	landSlideType = LANDSLIDE_GRAVITY;
+        landSlideDelay = MAX_GRAVITY_DELAY;
 	windstrength = 8;
 	windvariation = 1;
 	techLevel = 5;
@@ -37,6 +39,7 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 	lightning = 0;
 	fog = 0;
         wallType = 0.0;
+        current_wallType = 0;
 
 	done = new char[global->screenWidth];
 	fp = new int[global->screenWidth];
@@ -132,6 +135,39 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 #define GFILE_READ_VERSION_3    GFILE_VERSION_3(fread)
 #define GFILE_READ_VERSION_4    GFILE_VERSION_4(fread)
 
+
+/*
+This function saves the environment settings to a text
+file. Each line has the format
+name=value\n
+
+The function returns TRUE on success and FALSE on failure.
+-- Jesse
+*/
+int ENVIRONMENT::saveToFile_Text (FILE *file)
+{
+   if (! file) return FALSE;
+
+   fprintf(file, "*ENV*\n");
+
+   fprintf(file, "WINDVARIATION=%lf\n", windvariation);
+   fprintf(file, "VISCOSITY=%lf\n", viscosity);
+   fprintf(file, "GRAVITY=%lf\n", gravity);
+   fprintf(file, "TECHLEVEL=%lf\n", techLevel);
+   fprintf(file, "METEORS=%lf\n", meteors);
+   fprintf(file, "LIGHTNING=%lf\n", lightning);
+   fprintf(file, "FOG=%lf\n", fog);
+   fprintf(file, "LANDTYPE=%lf\n", landType);
+   fprintf(file, "LANDSLIDETYPE=%lf\n", landSlideType);
+   fprintf(file, "WALLTYPE=%lf\n", wallType);
+   fprintf(file, "LANDSLIDEDELAY=%lf\n", landSlideDelay);
+
+   fprintf(file, "***\n");      
+   return TRUE;
+}
+
+
+
 int ENVIRONMENT::saveToFile (FILE *file)
 {
 	int version = GFILE_LATEST_VERSION;
@@ -145,6 +181,93 @@ int ENVIRONMENT::saveToFile (FILE *file)
 	return (ferror (file));
 }
 
+
+
+/*
+This function loads environment settings from a text
+file. The function returns TRUE on success and FALSE if
+any erors are encountered.
+-- Jesse
+*/
+int ENVIRONMENT::loadFromFile_Text (FILE *file)
+{
+    char line[MAX_CONFIG_LINE];
+    int equal_position, line_length;
+    char field[MAX_CONFIG_LINE], value[MAX_CONFIG_LINE];
+    char *result = NULL;
+    bool done = false;
+
+    // read until we hit line "*ENV*" or "***" or EOF
+    do
+    {
+      result = fgets(line, MAX_CONFIG_LINE, file);
+      if (! result)     // eof
+         return FALSE;
+      if (! strncmp(line, "***", 3) )     // end of record
+         return FALSE;
+    } while ( strncmp(line, "*ENV*", 5) );     // read until we hit new record
+
+      while ( (result) && (!done) )
+      {
+       // read a line
+       memset(line, '\0', MAX_CONFIG_LINE);
+       result = fgets(line, MAX_CONFIG_LINE, file);
+       if (result)
+       {
+          // if we hit end of the record, stop
+          if (! strncmp(line, "***", 3) ) return TRUE;
+          // find equal sign
+          line_length = strlen(line);
+          // strip newline character
+          if ( line[line_length - 1] == '\n')
+          {
+               line[line_length - 1] = '\0';
+               line_length--;
+          }
+          equal_position = 1;
+          while ( ( equal_position < line_length) && (line[equal_position] != '=') )
+              equal_position++;
+          // make sure we have valid equal sign
+
+          if ( equal_position <= line_length )
+          {
+              // seperate field from value
+             memset(field, '\0', MAX_CONFIG_LINE);
+             memset(value, '\0', MAX_CONFIG_LINE);
+             strncpy(field, line, equal_position);
+             strcpy(value, & (line[equal_position + 1]));
+             // check for fields and values
+             if (! strcasecmp(field, "windvariation") )
+                sscanf(value, "%lf", &windvariation);
+             else if (! strcasecmp(field, "viscosity") )
+                sscanf(value, "%lf", &viscosity);
+             else if (! strcasecmp(field, "gravity") )
+                sscanf(value, "%lf", &gravity);
+             else if (! strcasecmp(field, "techlevel"))
+                sscanf(value, "%lf", &techLevel);
+             else if (! strcasecmp(field, "meteors"))
+                sscanf(value, "%lf", &meteors);
+             else if (! strcasecmp(field, "lightning") )
+                sscanf(value, "%lf", &lightning);
+             else if (! strcasecmp(field, "fog") )
+                sscanf(value, "%lf", &fog);
+             else if (! strcasecmp(field, "landtype"))
+                sscanf(value, "%lf", &landType);
+             else if (! strcasecmp(field, "landslidetype"))
+                sscanf(value, "%lf", &landSlideType);
+             else if (! strcasecmp(field, "walltype"))
+                sscanf(value, "%lf", &wallType);
+             else if (! strcasecmp(field, "landslidedelay"))
+                sscanf(value, "%lf", &landSlideDelay);
+          }    // end of found field=value line
+
+      }     // end of read a line properly
+     }     // end of while not done
+
+   return TRUE;
+}
+
+
 int ENVIRONMENT::loadFromFile (FILE *file)
 {
 	int version;
@@ -276,6 +399,15 @@ void ENVIRONMENT::newRound ()
 			}
 		}
 	}
+
+        // set wall type
+        if (wallType == WALL_RANDOM)
+           current_wallType = rand() % 3;
+        else
+           current_wallType = (int) wallType;
+
+        // time_to_fall = (rand() % MAX_GRAVITY_DELAY) + 1;
+        time_to_fall = (rand() & (int)landSlideDelay) + 1;
 }
 
 int ENVIRONMENT::ingamemenu ()
