@@ -19,6 +19,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * */
+#include "globaldata.h"
+
+
+#define PLAY_GAME 1
+#define LOAD_GAME 2
+#define ESC_MENU 3
+
 
 
 enum	menuEntryType {
@@ -49,4 +56,8 @@ typedef struct {
 	int okayButton;
 } MENUDESC;
 
+
+// Set the menus to appear in English or Portugese
+void Select_Menu_Language(GLOBALDATA *global);
+
 #endif
