@@ -184,7 +184,7 @@ int MISSILE::triggerTest ()
 						launchSpeed += weap->launchSpeed * variation;
 					}
 
-					newmis = new MISSILE (_global, _env, x, y - 20, _global->slope[dAngle][0] * launchSpeed * (100.0 / FRAMES_PER_SECOND) + inheritedXV, _global->slope[dAngle][1] * launchSpeed * (100.0 / FRAMES_PER_SECOND) + inheritedYV, weap->submunition);
+					newmis = new MISSILE (_global, _env, x, y - 20, _global->slope[dAngle][0] * launchSpeed * (100.0 / _global->frames_per_second) + inheritedXV, _global->slope[dAngle][1] * launchSpeed * (100.0 / _global->frames_per_second) + inheritedYV, weap->submunition);
 
 					if (!newmis) {
 						perror ("missile.cc: Failed allocating memory for newmis in MISSILE::triggerTest (CLUSTER)");
@@ -234,7 +234,7 @@ int MISSILE::applyPhysics ()
         else
            max_age = MAX_MISSLE_AGE;
 
-	if ( (age > max_age * FRAMES_PER_SECOND) || (y < -65535) )
+	if ( (age > max_age * _global->frames_per_second) || (y < -65535) )
 		detonate = 1;
 	if (!physics) {
 		if (type >= SML_METEOR && type <= LRG_METEOR) {
@@ -258,7 +258,11 @@ int MISSILE::applyPhysics ()
                    detonate = checkPixelsBetweenPrevAndNow ();
 		if (detonate &&
 			(type == BURROWER || type == PENETRATOR))
+                 {
 			hitSomething = 0;
+                        if ( ( y >= _global->screenHeight) && (_env->current_wallType == WALL_STEEL) )
+                             hitSomething = 1;
+                 }
 		if (rand () % 5 == 0) {
 			DECOR *decor;
 			decor = new DECOR (_global, _env, x, y, xv, yv, expSize / 20, DECOR_SMOKE);
@@ -270,7 +274,7 @@ int MISSILE::applyPhysics ()
 	} else {
 		if (type >= SML_ROLLER && type <= DTH_ROLLER) {
 			if (physics == 1) {
-				for (y = _global->screenHeight - 1; getpixel (_env->terrain, (int)x, (int)y) != PINK; y--);
+				// for (y = _global->screenHeight - 1; getpixel (_env->terrain, (int)x, (int)y) != PINK; y--);
 				if (getpixel (_env->terrain, (int)x + 1, (int)y + 1) == PINK)
 					physics = 3;
 				else
@@ -286,8 +290,9 @@ int MISSILE::applyPhysics ()
 			} else {
 				// roll roller
 				y += 3;
-				if ((y > 0) && ((y > _global->screenHeight - 1) || (getpixel (_env->terrain, (int)x, (int)y) != PINK)))
-					for (y = _global->screenHeight - 4; getpixel (_env->terrain, (int)x, (int)y + 3) != PINK; y--);
+				if ( (y > 0) && ( (y > _global->screenHeight - 1) || 
+                                    (getpixel (_env->terrain, (int)x, (int)y) != PINK) ) )
+					for (y = _global->screenHeight - 4; getpixel (_env->terrain, (int)x, (int)y + 3) != PINK; y--); 
 				if (y > (_global->screenHeight - 5) && !detonate)
 					y = (_global->screenHeight - 5);
 				if (angle < 0)
@@ -307,13 +312,13 @@ int MISSILE::applyPhysics ()
 			if (y + yv >= _global->screenHeight)
 			{
                               // only bounce if the wall is rubber
-                              if ( _env->wallType == WALL_RUBBER )
+                              if ( _env->current_wallType == WALL_RUBBER )
                               {
 			 	 yv = -yv * 0.5;
 				 xv *= 0.95;
                               }
                               // bounce back with more force
-                              else if ( _env->wallType == WALL_SPRING )
+                              else if ( _env->current_wallType == WALL_SPRING )
                               {
                                  yv = -yv * SPRING_CHANGE;
                                  xv *= 1.05;
@@ -340,13 +345,13 @@ int MISSILE::applyPhysics ()
 			}
 			if (x + xv < 1 || x + xv > (_global->screenWidth-1)) {
                              // if the wall is rubber, then bouce
-                             if ( _env->wallType == WALL_RUBBER )
+                             if ( _env->current_wallType == WALL_RUBBER )
           		         xv = -xv;	//bounce on the border
                              // bounce with more force
-                             else if ( _env->wallType == WALL_SPRING )
+                             else if ( _env->current_wallType == WALL_SPRING )
                                  xv = -xv * SPRING_CHANGE;
                              // wall is steel, detonate
-                             else
+                             else if ( _env->current_wallType == WALL_STEEL )
                                   detonate = TRUE;
 			}
 			if (y + yv >= _global->screenHeight) {
@@ -358,7 +363,7 @@ int MISSILE::applyPhysics ()
 			}
 			y += yv;
 			x += xv;
-			yv -= _env->gravity * 0.05 * (100.0 / FRAMES_PER_SECOND);
+			yv -= _env->gravity * 0.05 * (100.0 / _global->frames_per_second);
 			if (getpixel (_env->terrain, (int)x, (int)y) == PINK) {
 				detonate = TRUE;
 			}
