@@ -29,9 +29,9 @@ int PHYSICAL_OBJECT::applyPhysics ()
 	//motion - wind affected
 	if (x + xv < 1 || x + xv > (_global->screenWidth-1)) {
              // bounce on rubber
-             if ( _env->wallType == WALL_RUBBER )
+             if ( _env->current_wallType == WALL_RUBBER )
 		xv = -xv;	//bounce on the border
-             else if (_env->wallType == WALL_SPRING)
+             else if (_env->current_wallType == WALL_SPRING)
                 xv = -xv * SPRING_CHANGE;
              else // blow up on steel
                 hitSomething = 1;
@@ -45,14 +45,14 @@ int PHYSICAL_OBJECT::applyPhysics ()
 	if (y + yv >= _global->screenHeight)
 	{
               // rubber floor
-              if ( _env->wallType == WALL_RUBBER )
+              if ( _env->current_wallType == WALL_RUBBER )
               {
 		yv = -yv * 0.5;
 		xv *= 0.95;
 		if (fabs(xv) + fabs(yv) < 0.8)
 			hitSomething = 1;
               }
-              else if ( _env->wallType == WALL_SPRING )
+              else if ( _env->current_wallType == WALL_SPRING )
               {
                   yv = -yv * SPRING_CHANGE;
                   xv *= 1.05;
@@ -64,7 +64,7 @@ int PHYSICAL_OBJECT::applyPhysics ()
                   hitSomething = 1;
 	}
 	y += yv;
-	yv += _env->gravity * (100.0 / FRAMES_PER_SECOND);
+	yv += _env->gravity * (100.0 / _global->frames_per_second);
 
 	return (hitSomething);
 }
