@@ -24,13 +24,15 @@
 #include "tank.h"
 #include "menu.h"
 #include "lineseq.h"
-
+#include "files.h"
 
 // When defined draws AI 'planning'
 //#define AI_PLANNING_DEBUG	1
 
 PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 {
+        int my_colour;
+
 	_global = global;
 	_env = env;
 	money = 15000;
@@ -38,6 +40,7 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	played = 0;
 	won = 0;
 	tank = NULL;
+        team = TEAM_NEUTRAL;
 	_turnStage = 0;
 	selected = FALSE;
         changed_weapon = false;
@@ -66,7 +69,15 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	painSensitivity = (double)(rand () % 3000) / 1000;
 	defensive = (double)((rand () % 10000) - 5000) / 5000.0;
 
-	color = rand () % WHITE;
+	// color = rand () % WHITE;
+        my_colour = rand() % 4;
+        switch (my_colour)
+        {
+           case 0: color = makecol(200 + (rand() % 56), rand() % 25, rand() % 25); break;
+           case 1: color = makecol( rand() % 25, 200 + (rand() % 56), rand() % 25); break;
+           case 2: color = makecol( rand() % 25, rand() % 25, 200 + (rand() % 56) ); break;
+           case 3: color = makecol( 200 + (rand() % 56), rand() % 25, 200 + (rand() % 56)); break;
+        }
 	typeText[0] = "Human";
 	typeText[1] = "Useless";
 	typeText[2] = "Guesser";
@@ -77,10 +88,14 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	preftypeText[1] = "Only Once";
         tank_type[0] = "Normal";
         tank_type[1] = "Classic";
+        teamText[0] = "Sith";
+        teamText[1] = "Neutral";
+        teamText[2] = "Jedi";
+
 	initMenuDesc ();
 }
 
-PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
+PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file, bool text_file)
 {
 	_global = global;
 	_env = env;
@@ -91,6 +106,7 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
 	selected = FALSE;
         
         tank_bitmap = 0.0;
+        team = TEAM_NEUTRAL;
 
 	_targetMatrix = new double[global->screenWidth];
 
@@ -101,7 +117,18 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
 	for (int count = 0; count < ITEMS; count++)
 		ni[count] = 0;
 
-	loadFromFile (file);
+        // generate fresh weapon preferences because I goofed
+        // a while back and did not save them all. So to
+        // be backward compatible, generate the preferences.
+        // If they are saved, then the saved ones will
+        // over-write these ones
+        generatePreferences();
+
+	if (text_file)
+           loadFromFile_Text (file);
+        else
+           loadFromFile (file);
+
 	type = (int)type;
 
 	typeText[0] = "Human";
@@ -114,6 +141,9 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file)
 	preftypeText[1] = "Only Once";
         tank_type[0] = "Normal";
         tank_type[1] = "Classic";
+        teamText[0] = "Sith";
+        teamText[1] = "Neutral";
+        teamText[2] = "Jedi";
 	initMenuDesc ();
 }
 
@@ -167,6 +197,9 @@ void PLAYER::initMenuDesc ()
 {
 
 	int destroyPlayer (GLOBALDATA *global, ENVIRONMENT *env, void *data);
+        int i = 0;
+        int height = -68;
+
 	menudesc = new MENUDESC;
 	if (!menudesc) {
 		perror ("player.cc: Failed allocating memory for menudesc in PLAYER::initMenuDesc");
@@ -175,7 +208,7 @@ void PLAYER::initMenuDesc ()
 	menudesc->title = _name;
 
 	// Name,Color
-	menudesc->numEntries = 9;
+	menudesc->numEntries = 10;
 	menudesc->okayButton = TRUE;
 	menudesc->quitButton = FALSE;
 
@@ -186,105 +219,129 @@ void PLAYER::initMenuDesc ()
 	}
 
 	// Player name
-	menuopts[0].name = "Name";
-	menuopts[0].displayFunc = NULL;
-	menuopts[0].color = color;
-	menuopts[0].value = (double*)_name;
-	menuopts[0].specialOpts = NULL;
-	menuopts[0].type = OPTION_TEXTTYPE;
-	menuopts[0].viewonly = FALSE;
-	menuopts[0].x = _global->halfWidth - 3;
-	menuopts[0].y = _global->halfHeight - 68;
+	menuopts[i].name = "Name";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = color;
+	menuopts[i].value = (double*)_name;
+	menuopts[i].specialOpts = NULL;
+	menuopts[i].type = OPTION_TEXTTYPE;
+	menuopts[i].viewonly = FALSE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
 	// Player colour
-	menuopts[1].name = "Colour";
-	menuopts[1].displayFunc = NULL;
-	menuopts[1].color = WHITE;
-	menuopts[1].value = (double*)&color;
-	menuopts[1].specialOpts = NULL;
-	menuopts[1].type = OPTION_COLORTYPE;
-	menuopts[1].viewonly = FALSE;
-	menuopts[1].x = _global->halfWidth - 3;
-	menuopts[1].y = _global->halfHeight - 48;
+	menuopts[i].name = "Colour";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)&color;
+	menuopts[i].specialOpts = NULL;
+	menuopts[i].type = OPTION_COLORTYPE;
+	menuopts[i].viewonly = FALSE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
 	// Player type (human, computer)
-	menuopts[2].name = "Type";
-	menuopts[2].displayFunc = NULL;
-	menuopts[2].color = WHITE;
-	menuopts[2].value = (double*)&type;
-	menuopts[2].min = 0;
-	menuopts[2].max = LAST_PLAYER_TYPE - 1;
-	menuopts[2].increment = 1;
-	menuopts[2].defaultv = 0;
-	menuopts[2].format = "%s";
-	menuopts[2].specialOpts = typeText;
-	menuopts[2].type = OPTION_SPECIALTYPE;
-	menuopts[2].viewonly = FALSE;
-	menuopts[2].x = _global->halfWidth - 3;
-	menuopts[2].y = _global->halfHeight - 28;
-	
+	menuopts[i].name = "Type";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)&type;
+	menuopts[i].min = 0;
+	menuopts[i].max = LAST_PLAYER_TYPE - 1;
+	menuopts[i].increment = 1;
+	menuopts[i].defaultv = 0;
+	menuopts[i].format = "%s";
+	menuopts[i].specialOpts = typeText;
+	menuopts[i].type = OPTION_SPECIALTYPE;
+	menuopts[i].viewonly = FALSE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+	i++; height += 20;
+
+        menuopts[i].name = "Team";
+        menuopts[i].displayFunc = NULL;
+        menuopts[i].color = WHITE;
+        menuopts[i].value = (double *)&team;
+        menuopts[i].min = 0;
+        menuopts[i].max = TEAM_JEDI;
+        menuopts[i].increment = 1;
+        menuopts[i].defaultv = TEAM_NEUTRAL;
+        menuopts[i].format = "%s";
+        menuopts[i].specialOpts = teamText;
+        menuopts[i].type = OPTION_SPECIALTYPE;
+        menuopts[i].viewonly = FALSE;
+        menuopts[i].x = _global->halfWidth - 3;
+        menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
+
 	// Player preftype (human, computer)
-	menuopts[3].name = "Generate Pref.";
-	menuopts[3].displayFunc = NULL;
-	menuopts[3].color = WHITE;
-	menuopts[3].value = (double*)&preftype;
-	menuopts[3].min = 0;
-	menuopts[3].max = ALWAYS_PREF;
-	menuopts[3].increment = 1;
-	menuopts[3].defaultv = 0;
-	menuopts[3].format = "%s";
-	menuopts[3].specialOpts = preftypeText;
-	menuopts[3].type = OPTION_SPECIALTYPE;
-	menuopts[3].viewonly = FALSE;
-	menuopts[3].x = _global->halfWidth - 3;
-	menuopts[3].y = _global->halfHeight - 8;
+	menuopts[i].name = "Generate Pref.";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)&preftype;
+	menuopts[i].min = 0;
+	menuopts[i].max = ALWAYS_PREF;
+	menuopts[i].increment = 1;
+	menuopts[i].defaultv = 0;
+	menuopts[i].format = "%s";
+	menuopts[i].specialOpts = preftypeText;
+	menuopts[i].type = OPTION_SPECIALTYPE;
+	menuopts[i].viewonly = FALSE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
-	menuopts[4].name = "Played";
-	menuopts[4].displayFunc = NULL;
-	menuopts[4].color = WHITE;
-	menuopts[4].value = (double*)&played;
-	menuopts[4].format = "%.0f";
-	menuopts[4].specialOpts = NULL;
-	menuopts[4].type = OPTION_DOUBLETYPE;
-	menuopts[4].viewonly = TRUE;
-	menuopts[4].x = _global->halfWidth - 3;
-	menuopts[4].y = _global->halfHeight + 12;
+	menuopts[i].name = "Played";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)&played;
+	menuopts[i].format = "%.0f";
+	menuopts[i].specialOpts = NULL;
+	menuopts[i].type = OPTION_DOUBLETYPE;
+	menuopts[i].viewonly = TRUE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
-	menuopts[5].name = "Won";
-	menuopts[5].displayFunc = NULL;
-	menuopts[5].color = WHITE;
-	menuopts[5].value = (double*)&won;
-	menuopts[5].format = "%.0f";
-	menuopts[5].specialOpts = NULL;
-	menuopts[5].type = OPTION_DOUBLETYPE;
-	menuopts[5].viewonly = TRUE;
-	menuopts[5].x = _global->halfWidth - 3;
-	menuopts[5].y = _global->halfHeight + 32;
+	menuopts[i].name = "Won";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)&won;
+	menuopts[i].format = "%.0f";
+	menuopts[i].specialOpts = NULL;
+	menuopts[i].type = OPTION_DOUBLETYPE;
+	menuopts[i].viewonly = TRUE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
-        menuopts[6].name = "Tank Type";
-        menuopts[6].displayFunc = NULL;
-        menuopts[6].color = WHITE;
-        menuopts[6].value = &tank_bitmap;
-        menuopts[6].min = 0;
-        menuopts[6].max = 1;
-        menuopts[6].increment = 1;
-        menuopts[6].defaultv = 0;
-        menuopts[6].format = "%1.0f";
-        menuopts[6].specialOpts = tank_type;
-        menuopts[6].type = OPTION_SPECIALTYPE;
-        menuopts[6].viewonly = FALSE;
-        menuopts[6].x = _global->halfWidth - 3;
-        menuopts[6].y = _global->halfHeight + 52;
+        menuopts[i].name = "Tank Type";
+        menuopts[i].displayFunc = NULL;
+        menuopts[i].color = WHITE;
+        menuopts[i].value = &tank_bitmap;
+        menuopts[i].min = 0;
+        menuopts[i].max = 1;
+        menuopts[i].increment = 1;
+        menuopts[i].defaultv = 0;
+        menuopts[i].format = "%1.0f";
+        menuopts[i].specialOpts = tank_type;
+        menuopts[i].type = OPTION_SPECIALTYPE;
+        menuopts[i].viewonly = FALSE;
+        menuopts[i].x = _global->halfWidth - 3;
+        menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
-	menuopts[8].name = "Delete This Player";
-	menuopts[8].displayFunc = NULL;
-	menuopts[8].color = WHITE;
-	menuopts[8].value = (double*)destroyPlayer;
-	menuopts[8].data = (void*)this;
-	menuopts[8].type = OPTION_ACTIONTYPE;
-	menuopts[8].viewonly = FALSE;
-	menuopts[8].x = _global->halfWidth - 3;
-	menuopts[8].y = _global->halfHeight + 82;
+	menuopts[i].name = "Delete This Player";
+	menuopts[i].displayFunc = NULL;
+	menuopts[i].color = WHITE;
+	menuopts[i].value = (double*)destroyPlayer;
+	menuopts[i].data = (void*)this;
+	menuopts[i].type = OPTION_ACTIONTYPE;
+	menuopts[i].viewonly = FALSE;
+	menuopts[i].x = _global->halfWidth - 3;
+	menuopts[i].y = _global->halfHeight + height;
+        i++; height += 20;
 
 	menudesc->entries = menuopts;
 }
@@ -460,6 +517,147 @@ PLAYER::~PLAYER ()
 #define GFILE_READ_VERSION_11   GFILE_VERSION_11(fread);
 
 
+
+/*
+Save the player settings in text form. Fields are
+formateed as
+[name]=[value]\n
+
+Function returns true on success and false on failure.
+*/
+int PLAYER::saveToFile_Text (FILE *file)
+{
+   int count;
+
+   if (! file) return FALSE;
+   // start section with "*PLAYER*"
+   fprintf(file, "*PLAYER*\n");
+   fprintf(file, "NAME=%s\n", _name);
+   fprintf(file, "VENGEFUL=%d\n", vengeful);
+   fprintf(file, "VENGEANCETHRESHOLD=%lf\n", vengeanceThreshold);
+   fprintf(file, "TYPE=%lf\n", type);
+   fprintf(file, "COLOR=%d\n", color);
+   fprintf(file, "COLOR2=%d\n", color2);
+   for (count = 0; count < THINGS; count++)
+      fprintf(file, "WEAPONPREFERENCES=%d %d\n", count, _weaponPreference[count]);
+
+   fprintf(file, "PLAYED=%lf\n", played);
+   fprintf(file, "WON=%lf\n", won);
+   fprintf(file, "PREFTYPE=%lf\n", preftype);
+   fprintf(file, "SELFPRESERVATION=%lf\n", selfPreservation);
+   fprintf(file, "PAINSENSITIVITY=%lf\n", painSensitivity);
+   fprintf(file, "DEFENSIVE=%lf\n", defensive);
+   fprintf(file, "TANK_BITMAP=%lf\n", tank_bitmap);
+   fprintf(file, "TEAM=%lf\n", team);
+   fprintf(file, "***\n");
+   return TRUE;
+}
+
+
+/*
+This function tries to load player data from a text file.
+Each line is parsed for "field=value", except WEAPONPREFERENCES
+which is parsed "field=index value".
+If all goes well TRUE is returned, on error the function returns FALSE.
+-- Jesse
+*/
+int PLAYER::loadFromFile_Text (FILE *file)
+{
+    char line[MAX_CONFIG_LINE];
+    int equal_position, line_length;
+    int index, wp_value;
+    char field[MAX_CONFIG_LINE], value[MAX_CONFIG_LINE];
+    char *result = NULL;
+    bool done = false;
+
+    if (! file) return FALSE;
+
+    // read until we hit line "*PLAYER*" or "***" or EOF
+    do
+    {
+      result = fgets(line, MAX_CONFIG_LINE, file);
+      if (! result)     // eof
+         return FALSE;
+      if (! strncmp(line, "***", 3) )     // end of record
+         return FALSE;
+    } while ( strncmp(line, "*PLAYER*", 8) );     // read until we hit new record
+
+    while ( (result) && (!done) )
+    {
+       // read a line
+       memset(line, '\0', MAX_CONFIG_LINE);
+       result = fgets(line, MAX_CONFIG_LINE, file);
+       if (result)
+       {
+          // if we hit end of the record, stop
+          if (! strncmp(line, "***", 3) ) return TRUE;
+          // find equal sign
+          line_length = strlen(line);
+          // strip newline character
+          if ( line[line_length - 1] == '\n')
+          {
+               line[line_length - 1] = '\0';
+               line_length--;
+          }
+          equal_position = 1;
+          while ( ( equal_position < line_length) && (line[equal_position] != '=') )
+              equal_position++;
+          // make sure we have valid equal sign
+          if (equal_position <= line_length)
+          {
+             // seperate field from value
+             memset(field, '\0', MAX_CONFIG_LINE);
+             memset(value, '\0', MAX_CONFIG_LINE);
+             strncpy(field, line, equal_position);
+             strcpy(value, & (line[equal_position + 1]));
+             // check which field we have and process value
+             if (! strcasecmp(field, "name") )
+                 strncpy(_name, value, NAME_LENGTH);
+             else if (! strcasecmp(field, "vengeful") )
+                 sscanf(value, "%d", &vengeful);
+             else if (! strcasecmp(field, "vengeancethreshold") )
+                 sscanf(value, "%lf", &vengeanceThreshold);
+             else if (! strcasecmp(field, "type") )
+                 sscanf(value, "%lf", &type);
+             else if (! strcasecmp(field, "color") )
+                 sscanf(value, "%d", &color);
+             else if (! strcasecmp(field, "color2") )
+                 sscanf(value, "%d", &color2);
+             else if (! strcasecmp(field, "played") )
+                 sscanf(value, "%lf", &played);
+             else if (! strcasecmp(field, "won") )
+                 sscanf(value, "%lf", &won);
+             else if (! strcasecmp(field, "preftype") )
+                 sscanf(value, "%lf", &preftype);
+             else if (! strcasecmp(field, "selfpreservation") )
+                 sscanf(value, "%lf", &selfPreservation);
+             else if (! strcasecmp(field, "painsensititveity") )
+                 sscanf(value, "%lf", &painSensitivity);
+             else if (! strcasecmp(field, "defensive") )
+                 sscanf(value, "%lf", &defensive);
+             else if (! strcasecmp(field, "tank_bitmap") )
+                 sscanf(value, "%lf", &tank_bitmap);
+             else if (! strcasecmp(field, "team") )
+                 sscanf(value, "%lf", &team);
+             else if (! strcasecmp(field, "weaponpreferences") )
+             {
+                 sscanf(value, "%d %d", &index, &wp_value);
+                 if ( (index < THINGS) && (index >= 0) )
+                    _weaponPreference[index] = wp_value;
+             }
+
+          }    // end of valid data line
+       }      // end of if we read a line properly
+    }   // end of while not done
+
+     // make sure previous human players are restored as humans
+    if (type == PART_TIME_BOT)
+           type = HUMAN_PLAYER;
+
+    return TRUE;
+}
+
+
 int PLAYER::saveToFile (FILE *file)
 {
 	int version = GFILE_LATEST_VERSION;
@@ -570,7 +768,7 @@ void PLAYER::exitShop ()
 void PLAYER::newRound ()
 {
         // if the player is under computer control, give it back to the player
-        if ( type == PART_TIME_BOT)
+        if ( type == PART_TIME_BOT )
            type = HUMAN_PLAYER;
 
 	setComputerValues ();
@@ -584,15 +782,16 @@ void PLAYER::newRound ()
 	}
 	tank->newRound ();
 
-        /* I am not thrilled about putting this here, but we need a
-        way to avoid having too much money or too little (haha) money.
+        /*
+        This used to make sure money did not over-flow.
+        But the check has been moved to the buystuff function
+        because interest is applied there. 
         -- Jesse
-        */
         if (money > 1000000000)
            money = 1000000000;
         if (money < 0)
            money = 0;
-
+        */
         changed_weapon = false;
 }
 
@@ -667,7 +866,7 @@ int PLAYER::controlTank (TANK *ctank)
 	}
 
 	if (_global->computerPlayersOnly &&
-		((int)_global->skipComputerPlay >= SKIP_AUTOPLAY)) {
+		((int)_global->skipComputerPlay >= SKIP_HUMANS_DEAD)) {
 		if (_env->stage == 3)
 			return (-1);
 	}
@@ -685,8 +884,7 @@ int PLAYER::controlTank (TANK *ctank)
 			void clockadd ();
 			install_int_ex (clockadd, SECS_TO_TIMER(6000));
 			int mm = _env->ingamemenu ();
-			// install_int_ex (clockadd, BPS_TO_TIMER (100));
-                        install_int_ex(clockadd, BPS_TO_TIMER(FRAMES_PER_SECOND));
+                        install_int_ex(clockadd, BPS_TO_TIMER(_global->frames_per_second));
 			_env->make_update (0, 0, _global->screenWidth, _global->screenHeight);
 			_env->make_bgupdate (0, 0, _global->screenWidth, _global->screenHeight);
 			
@@ -702,6 +900,31 @@ int PLAYER::controlTank (TANK *ctank)
 				return (-1);
 			}
 		}
+                // check for number key being pressed
+                if ( (k >> 8 >= KEY_0) && (k >> 8 <= KEY_9) )
+                {
+                    int value = (k >> 8) - KEY_0;
+                   
+                    // make sure the value is within range
+                    if (value < _global->numPlayers)
+                    {
+                        if ( _global->players[value] )
+                        {
+                            TANK *my_tank = _global->players[value]->tank;
+                            if (my_tank)
+                            { 
+                                sprintf(_global->tank_status, "%s: %d + %d -- Team: ", _global->players[value]->_name,
+                                        my_tank->l, my_tank->sh);
+                                strcat(_global->tank_status, _global->players[value]->Get_Team_Name());
+                                _global->tank_status_colour = _global->players[value]->color;
+                                _global->updateMenu = 1;
+                            }
+                            else
+                              _global->tank_status[0] = 0;
+                        }
+                    }
+                       
+                }    // end of check status keys
 	}
 	if ((int)type == HUMAN_PLAYER || !ctank) {
 		return (humanControls (ctank));
@@ -713,11 +936,31 @@ int PLAYER::controlTank (TANK *ctank)
 
 int PLAYER::chooseItemToBuy ()
 {
+	int currItem = selectRandomItem ();
+	int itemNum = currItem - WEAPONS;
+	int cumulative;
+	int nextMod,curramt,newamt;
+
+        /* if the AI is intelligent and there is gravity,
+           they buy parachutes
+        */
+        if ( ( (type >= RANGEFINDER_PLAYER) && (type <= DEADLY_PLAYER) ) &&
+             ( _env->landSlideType > LANDSLIDE_NONE))
+        {
+            if ( ni[ITEM_PARACHUTE] < 10)
+            { 
+               // do we have the funds?
+               if (money >= item[ITEM_PARACHUTE].cost )
+               {
+                     money -= item[ITEM_PARACHUTE].cost;
+                     ni[ITEM_PARACHUTE] += item[ITEM_PARACHUTE].amt;
+                     return ITEM_PARACHUTE;
+               }
+            }
+        }
+
 	//loop through all weapon preferences
 	for (int count = 0; count < THINGS; count++) {
-		int currItem = selectRandomItem ();
-		int itemNum = currItem - WEAPONS;
-		int cumulative;
 
 		//is item available for purchase or selected first item
 		//first item is free
@@ -727,7 +970,6 @@ int PLAYER::chooseItemToBuy ()
 		//determine the likelyhood of purchasing this selection
 		//less likely the more of the item is owned		
 		//if have zero of item, it is a fifty/fifty chance of purchase	
-		int nextMod,curramt,newamt;
 		if(currItem < WEAPONS) {
 		  curramt = nm[currItem];
 		  newamt = weapon[currItem].amt;
@@ -738,6 +980,7 @@ int PLAYER::chooseItemToBuy ()
 
 		if ((itemNum >= ITEM_INTENSITY_AMP &&
 			itemNum <= ITEM_VIOLENT_FORCE) ||
+                        (itemNum == ITEM_REPAIRKIT) ||
 			(itemNum >= ITEM_ARMOUR &&
 			 itemNum <= ITEM_PLASTEEL)) {
 			cumulative = TRUE;
@@ -754,16 +997,7 @@ int PLAYER::chooseItemToBuy ()
 			nextMod = 1;
 		}
 
-		//if amount available for purchuse is more then current amount
-		//already bought 33% chance of purchase
-		/*if(newamt > curramt)
-			nextMod = 3;
-		else if (curramt == 0) //don't have any 50% chance of purchase
-			nextMod = 2;
-		else //have at one/more purchase full 5% chance of purchase
-			nextMod = 20;*/
-		
-		if (rand () % nextMod)
+		if (rand () % nextMod) 
 			continue;
 			
 		//weapon
@@ -788,7 +1022,7 @@ int PLAYER::chooseItemToBuy ()
 			//purchase the item
 			if (money >= item[currItem - WEAPONS].cost) {
 				money -= item[currItem - WEAPONS].cost;
-				ni[currItem - WEAPONS] += item[currItem- WEAPONS].amt;
+				ni[currItem - WEAPONS] += item[currItem - WEAPONS].amt;
 				//don't allow more than 99
 				if(ni[currItem - WEAPONS] > 99) ni[currItem - WEAPONS] = 99;
 				return (currItem);
@@ -798,21 +1032,70 @@ int PLAYER::chooseItemToBuy ()
 	return (-1);
 }
 
-const char *PLAYER::selectRevengePhrase (double scale)
+char *PLAYER::selectRevengePhrase (double scale)
 {
-	static const LINESEQ says( DATA_DIR "/revenge.txt", LINESEQ::skip_blanks );
+        char *filename;
+        char *line;
 
-	return says.random( );
+        filename = (char *) calloc( strlen(DATA_DIR) + 64, sizeof(char) );
+        if (! filename) return NULL;
+
+        strcpy(filename, DATA_DIR);
+        if (_global->language == LANGUAGE_ENGLISH)
+           strcat(filename, "/revenge.txt");
+        else if (_global->language == LANGUAGE_PORTUGUESE)
+           strcat(filename, "/revenge.pt_BR.txt");
+
+        // static const LINESEQ says( filename, LINESEQ::skip_blanks );
+        line = Random_Line(filename);
+        free(filename);
+        return line;
+	// return says.random( );
 }
 
-const char *PLAYER::selectGloatPhrase (double scale)
+char *PLAYER::selectGloatPhrase (double scale)
 {
-	static const LINESEQ says( DATA_DIR "/gloat.txt", LINESEQ::skip_blanks );
+        char *filename;
+        char *line;
 
-	return says.random( );
+        filename = (char *) calloc( strlen(DATA_DIR) + 64, sizeof(char) );
+        if (! filename) return NULL;
 
+        strcpy(filename, DATA_DIR);
+	if (_global->language == LANGUAGE_ENGLISH)
+           strcat(filename, "/gloat.txt");
+        else if (_global->language == LANGUAGE_PORTUGUESE)
+           strcat(filename, "/gloat.pt_BR.txt");
+
+        // static const LINESEQ says( filename, LINESEQ::skip_blanks );
+        line = Random_Line(filename);
+        free(filename); 
+        return line;
+	// return says.random( );
 }
 
+char *PLAYER::selectSuicidePhrase (double scale)
+{
+        char *filename;
+        char *line;
+
+        filename = (char *) calloc( strlen(DATA_DIR) + 64, sizeof(char) );
+        if (! filename) return NULL;
+
+        strcpy(filename, DATA_DIR);
+        if (_global->language == LANGUAGE_ENGLISH)
+           strcat(filename, "/suicide.txt");
+        else if (_global->language == LANGUAGE_PORTUGUESE)
+           strcat(filename, "/suicide.pt_BR.txt");
+
+        // static const LINESEQ says( filename, LINESEQ::skip_blanks );
+        line = Random_Line(filename);       
+        free(filename);
+        return line;
+        // return says.random();
+}
+
+
 int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, double targetX, double targetY, double x, double y, double xv, double yv)
 {
 	TANK *ltank;
@@ -847,18 +1130,21 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
                 // hit side of the screen
 		if (x + xv < 1 || x + xv > (_global->screenWidth-1)) {
                       // bounce if wall is rubber
-                      if ( _env->wallType == WALL_RUBBER )
+                      if ( _env->current_wallType == WALL_RUBBER )
                       {
 		  	  xv = -xv;	//bounce on the border
 			  direction = -direction;
                       }
-                      else if ( _env->wallType == WALL_SPRING )
+                      else if ( _env->current_wallType == WALL_SPRING )
                       {
                           xv = -xv * SPRING_CHANGE;
                           direction = -direction; 
                       }
-                      else   // wall is steel
+                      else if ( _env->current_wallType == WALL_STEEL )  // wall is steel
+                      {
                           hitSomething = 1;
+                          break;
+                      }
 		} else {
 			// motion - wind affected
 			double accel = (_env->wind - xv) / weapon[ctank->cw].mass * drag * _env->viscosity;
@@ -873,7 +1159,7 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
 				hitSomething = 1;
 		}
 		y += yv;
-		yv += _env->gravity * (100.0 / FRAMES_PER_SECOND);
+		yv += _env->gravity * (100.0 / _global->frames_per_second);
 
 		if (	(y > _global->screenHeight + 100) ||
 			(x > _global->screenWidth + 100) ||
@@ -889,7 +1175,7 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
 		}
 #ifdef	AI_PLANNING_DEBUG
 		// Plot trajectories for debugging purposes
-		//circlefill (screen, (int)x, (int)y, 2, color);
+		circlefill (screen, (int)x, (int)y, 2, color);
 #endif
 		ticks++;
 	}
@@ -926,8 +1212,8 @@ int PLAYER::rangeFind (TANK *ctank, int *estimateX, int *estimateY, int maxAttem
 			int ca;
 
 			ca = _targetAngle + ((SPREAD * z) - (SPREAD * (spread - 1) / 2));
-			mxv = _global->slope[ca][0] * _targetPower * (100.0 / FRAMES_PER_SECOND) / 100;
-			myv = _global->slope[ca][1] * _targetPower * (100.0 / FRAMES_PER_SECOND) / 100;
+			mxv = _global->slope[ca][0] * _targetPower * (100.0 / _global->frames_per_second) / 100;
+			myv = _global->slope[ca][1] * _targetPower * (100.0 / _global->frames_per_second) / 100;
 
 			overshoot = (int)(traceShellTrajectory (
 				ctank,
@@ -1010,10 +1296,9 @@ int PLAYER::calculatePowerGivenAngle (int dx, int dy, int angle)
 	*/
 
 
-	//maxHeight = 0.5 * (yv * yv) / _env->gravity * (100.0 / FRAMES_PER_SECOND);
 	// calculate minimum V to reach height dy
 	if (dy > 0)
-		minYv = sqrt (dy * _env->gravity * 2 * (100.0 / FRAMES_PER_SECOND));
+		minYv = sqrt (dy * _env->gravity * 2 * (100.0 / _global->frames_per_second));
 	else
 		minYv = 0;
 	if ((slopeY != 0) && (minYv != 0))
@@ -1037,7 +1322,7 @@ int PLAYER::calculatePowerGivenAngle (int dx, int dy, int angle)
 	}
 	// Low target, less power
 	// xdistance proportional to sqrt(dy)
-	airTime = fabs(dxTime) + ((dy * slopeY) * _env->gravity * (100.0 / FRAMES_PER_SECOND)) * 2;
+	airTime = fabs(dxTime) + ((dy * slopeY) * _env->gravity * (100.0 / _global->frames_per_second)) * 2;
 
 	// Less airTime doesn't necessarily mean less power
 	// Horizontal firing means more power needed even though
@@ -1046,7 +1331,7 @@ int PLAYER::calculatePowerGivenAngle (int dx, int dy, int angle)
 //	powerMult = (slopeX * slopeY);
 //	if (powerMult == 0)
 //		powerMult = 0.01;
-	requiredPower = (sqrt (airTime * _env->gravity * (100.0 / FRAMES_PER_SECOND)) / powerMult) * 100;
+	requiredPower = (sqrt (airTime * _env->gravity * (100.0 / _global->frames_per_second)) / powerMult) * 100;
 	requiredPower += (int)((rand () % 100 - 50) * errorMultiplier);
 	return ((int)requiredPower);
 }
@@ -1109,7 +1394,10 @@ int PLAYER::addTankDamageToMatrix (TANK *ctank, TANK *ltank, int weapNum)
 		windAffect = 0;
 	}
 
-	if (ltank->player == this) {
+	if ( (ltank->player == this) ||
+             ( (ltank->player->team == team) &&
+               (team != TEAM_NEUTRAL) ) )
+        {
 		wdamage = -wdamage * painSensitivity;
 		radius = (int)(radius * selfPreservation);
 	} else if (ltank->player == revenge) {
@@ -1605,3 +1893,21 @@ int PLAYER::humanControls (TANK *ctank)
 	}
 	return (0);
 }
+
+
+
+// returns a static string to the player's team name
+char *PLAYER::Get_Team_Name()
+{
+    char *team_name = "";
+
+    switch ( (int) team)
+    {
+       case TEAM_JEDI: team_name = "Jedi"; break;
+       case TEAM_NEUTRAL: team_name = "Neutral"; break;
+       case TEAM_SITH: team_name = "Sith"; break;
+    }
+
+    return team_name;
+}
+
