@@ -85,13 +85,13 @@ TANK::TANK (GLOBALDATA *global, ENVIRONMENT *env)
 {
 	setEnvironment (env);
 	player = NULL;
-	healthText = new FLOATTEXT (global, env, "", 0, 0, WHITE, CENTRE);
+	healthText = new FLOATTEXT (global, env, NULL, 0, 0, WHITE, CENTRE);
 	if (!healthText) {
 		perror ("tank.cc: Failed allocating memory for healthText in TANK::TANK");
 		exit (1);
 	}
 
-        shieldText = new FLOATTEXT (global, env, "", 0, 0, makecol (200, 200, 255), CENTRE);
+        shieldText = new FLOATTEXT (global, env, NULL, 0, 0, makecol (200, 200, 255), CENTRE);
 	if (!shieldText) {
 		perror ("tank.cc: Failed allocating memory for shieldText in TANK::TANK");
 		exit (1);
@@ -99,7 +99,7 @@ TANK::TANK (GLOBALDATA *global, ENVIRONMENT *env)
 
         if (global->name_above_tank)
         {
-            nameText = new FLOATTEXT (global, env, "", 0, 0, WHITE, CENTRE);
+            nameText = new FLOATTEXT (global, env, NULL, 0, 0, WHITE, CENTRE);
             if (! nameText)
             {
                 perror ("tank.cc: Failed allocating memory for nameText in TANK::TANK");
@@ -172,6 +172,8 @@ void TANK::newRound ()
 	fs = 0;
 	sprintf (buf, "%d", l);
 	healthText->set_text (buf);
+        // delay_fall = GRAVITY_DELAY;
+        delay_fall = (int) _env->landSlideDelay * 100;
 
 }
 
@@ -191,26 +193,35 @@ void TANK::applyDamage ()
 	FLOATTEXT *damageText;
 	FLOATTEXT *revengeText;
         FLOATTEXT *gloatText;
+        FLOATTEXT *suicideText;
+        char *temp_text;
+        bool killed = false;
 
 	if (damage > sh + l)
-		damage = sh + l;
+        {
+	    damage = sh + l;
+            killed = true;
+        }
 	sh -= (int)damage;
 	if (creditTo) {
 		if (player != creditTo) {	//enemy hit ++
 			creditTo->money += (int)(damage * _global->scoreHitUnit);
                         // this tank is destroyed, the attacker gloats
-                        if ( (damage >= sh + l) && (! creditTo->gloating) )
+                        if ( (killed) && (! creditTo->gloating) )
                         {
                            // avoid trying to print victory message over a dead tank
                            if (creditTo->tank)
                            {
+                                temp_text = creditTo->selectGloatPhrase( (double) damage/ maxLife);
                                 gloatText = new FLOATTEXT (_global, _env,
-                                        creditTo->selectGloatPhrase( (double) damage / maxLife),
+                                        // creditTo->selectGloatPhrase( (double) damage / maxLife),
+                                        temp_text,
                                         (int) creditTo->tank->x, (int) creditTo->tank->y - 30,
                                         creditTo->color, CENTRE);
+                                if (temp_text) free(temp_text);
                                 if (! gloatText)
                                 {
-                                   perror ("tank.cc: Failed allocating memory for gloatTExt in applyDamage.");
+                                   perror ("tank.cc: Failed allocating memory for gloatText in applyDamage.");
                                    exit(1);
                                 }
                                 gloatText->xv = 0;
@@ -228,10 +239,13 @@ void TANK::applyDamage ()
 				}
 				if (player->annoyanceFactor > player->vengeanceThreshold * maxLife) {
 					player->revenge = creditTo;
+                                        temp_text = player->selectRevengePhrase( (double) damage / maxLife);
 					revengeText = new FLOATTEXT (_global, _env,
-						player->selectRevengePhrase ((double)damage / maxLife),
+						// player->selectRevengePhrase ((double)damage / maxLife),
+                                                temp_text,
 						(int)x, (int)y - 30,
 						player->color, CENTRE);
+                                        if (temp_text) free(temp_text);
 					if (!revengeText) {
 						perror ("tank.cc: Failed allocating memory for revengeText in applyDamage");
 						exit (1);
@@ -246,8 +260,24 @@ void TANK::applyDamage ()
                            creditTo->money = 0;
                         else
 			   creditTo->money -= (int)(damage * _global->scoreSelfHit);
-			// if (creditTo->money < 0)
-			//	creditTo->money = 0;
+
+                        if (damage >= (l + sh) )
+                        {
+                            temp_text = player->selectSuicidePhrase( (double) damage / maxLife);
+                            suicideText = new FLOATTEXT (_global, _env,
+                                        player->selectSuicidePhrase( (double) damage / maxLife),
+                                        (int) x, (int) y - 30,
+                                        player->color, CENTRE);
+                            if (temp_text) free(temp_text);
+                            if (! suicideText)
+                            {
+                                perror ("tank.cc: Failed allocating memory for suicideText in applyDamage.");
+                                exit(1);
+                            }
+                            suicideText->xv = 0;
+                            suicideText->yv = -0.4;
+                            suicideText->maxAge = 300;
+                       }
 		}
 	}
 
@@ -266,12 +296,13 @@ void TANK::applyDamage ()
 		flashdamage = 1;
 		sprintf (buf, "%d", (int)damage);
 		damageText = new FLOATTEXT (_global, _env,
-				buf, (int)x, (int)y,
+				NULL, (int)x, (int)y,
 				makecol (255, 0, 0), CENTRE);
 		if (!damageText) {
 			perror ("tank.cc: Failed allocating memory for damageText in TANK::TANK");
 			exit (1);
 		}
+                damageText->set_text(buf);
 		damageText->xv = 0;
 		damageText->yv = -0.2;
 		damageText->maxAge = 300;
@@ -324,18 +355,28 @@ int TANK::applyPhysics (GLOBALDATA *global)
 			if (y > _global->screenHeight - TANKHEIGHT)
 				y = _global->screenHeight - TANKHEIGHT;
 
-		} 
-                else if ((y < _global->screenHeight - TANKHEIGHT) && (pixelCol == PINK) && (l > 0) && (! landed_on_tank))
+                        // delay_fall = GRAVITY_DELAY;
+                        delay_fall = (int) _env->landSlideDelay * 100;
+
+		}
+
+                // the tank is falling 
+                else if ((y < _global->screenHeight - TANKHEIGHT) && (pixelCol == PINK) && (l > 0) && 
+                         (! landed_on_tank) && (_env->landSlideType > LANDSLIDE_NONE) )
                 {
+                        delay_fall--;
+                        if ( (delay_fall > 0) && (_env->landSlideType == LANDSLIDE_CARTOON) )
+                           return stable;
+
 			if (para && para < 3 && !_env->pclock)
 				para++;
 			if (!para) {
-				yv += _env->gravity * (100.0 / FRAMES_PER_SECOND);
+				yv += _env->gravity * (100.0 / _global->frames_per_second);
 				y += yv;
 			} else {
 				double accel = (_env->wind - xv) / mass * (drag + 0.35) * _env->viscosity;
 				xv += accel;
-				yv += _env->gravity * (100.0 / FRAMES_PER_SECOND);
+				yv += _env->gravity * (100.0 / _global->frames_per_second);
 				if (yv > 0.5 )
 					yv = 0.5;
 				x += xv;
@@ -370,13 +411,17 @@ void TANK::explode ()
 {
 	FLOATTEXT *revengeText;
 	EXPLOSION *explosion;
+        char *temp_text;
 
 	if ((int)player->type != HUMAN_PLAYER) {
 		player->revenge = creditTo;
+                temp_text = player->selectRevengePhrase( (double) damage / maxLife);
 		revengeText = new FLOATTEXT (_global, _env,
-			player->selectRevengePhrase ((double)damage / maxLife),
+			// player->selectRevengePhrase ((double)damage / maxLife),
+                        temp_text,
 			(int)x, (int)y - 30,
 			player->color, CENTRE);
+                if (temp_text) free(temp_text);
 		if (!revengeText) {
 			perror ("tank.cc: Failed allocating memory for revengeText in TANK::explode");
 			exit (1);
@@ -456,12 +501,15 @@ void TANK::draw (BITMAP *dest, int healthOffset)
 		(int) (x + TANKWIDTH) - 2,
 		(int) (y + TANKHEIGHT),
 		this->player->color);
-	draw_sprite (dest, (BITMAP *) _global->gfxData.T[use_tank_bitmap].dat, (int) x - TANKWIDTH, (int) y);
+	// draw_sprite (dest, (BITMAP *) _global->gfxData.T[use_tank_bitmap].dat, (int) x - TANKWIDTH, (int) y);
 	if (sh > 0) {
 		int phaseValue = 1;
 
 		drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
 		set_trans_blender (0, 0, 0, (int)50);
+                // avoid sub-index
+                if ( shPhase < 0 )
+                   shPhase = 0;
 		if (sht >= ITEM_LGT_SHIELD && sht <= ITEM_HVY_SHIELD) {
 			phaseValue = (int)(_global->slope[(int)shPhase][0] * 3);
 		} else if (sht >= ITEM_LGT_REPULSOR_SHIELD && sht <= ITEM_HVY_REPULSOR_SHIELD) {
@@ -472,10 +520,12 @@ void TANK::draw (BITMAP *dest, int healthOffset)
 		set_trans_blender (0, 0, 0, (int)
 				   (50 + (((float) sh / (float) (item[sht]).vals[SHIELD_ENERGY]) * (float) 150)));
 		for (int thicknessCount = 0; thicknessCount < shieldThickness; thicknessCount++) {
-			ellipse (dest, (int) x - (shieldThickness / 2) + thicknessCount, (int) y, TANKWIDTH + 6 + phaseValue, TANKHEIGHT - 1, shieldColor);
+			ellipse (dest, (int) x - (shieldThickness / 2) + thicknessCount, (int) y, 
+                                 TANKWIDTH + 6 + phaseValue, TANKHEIGHT - 1, shieldColor);
 		}
 		drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
 	}
+        draw_sprite (dest, (BITMAP *) _global->gfxData.T[use_tank_bitmap].dat, (int) x - TANKWIDTH, (int) y);
 	turretAngle = (int) ((float) (90 - a) * ((float) 256 / (float) 360));
 	rotate_sprite (dest, (BITMAP *) _global->gfxData.TG[use_turret_bitmap].dat,
 		       (int) turret_x - GUNLENGTH, (int) turret_y - (GUNLENGTH - 2), itofix (turretAngle));
@@ -558,6 +608,14 @@ void TANK::activateCurrentSelection ()
              (_global->command == GLOBAL_COMMAND_MENU) )
            return;
 
+        // remove status from top bar at next redraw
+        _global->tank_status[0] = 0;
+
+        _env->time_to_fall--;
+        if (_env->time_to_fall < 0)
+           // _env->time_to_fall = (rand() % MAX_GRAVITY_DELAY) + 1;
+           _env->time_to_fall = (rand() % (int)_env->landSlideDelay) + 1;
+
 	if (cw < WEAPONS) {
                 player->changed_weapon = false;
 		if (cw)
@@ -574,8 +632,8 @@ void TANK::activateCurrentSelection ()
 				int ca;
 
 				ca = a + ((SPREAD * z) - (SPREAD * (_env->am - 1) / 2));
-				mxv = _global->slope[ca][0] * p * (100.0 / FRAMES_PER_SECOND) / 100;
-				myv = _global->slope[ca][1] * p * (100.0 / FRAMES_PER_SECOND) / 100;
+				mxv = _global->slope[ca][0] * p * (100.0 / _global->frames_per_second) / 100;
+				myv = _global->slope[ca][1] * p * (100.0 / _global->frames_per_second) / 100;
 
 				newmis = new MISSILE(_global, _env,
 					turret_x + (_global->slope[ca][0] * GUNLENGTH) /*- mxv*/,
