@@ -1,3 +1,6 @@
+#ifndef WEAPONS_HEADER_
+#define WEAPONS_HEADER_
+
 /*
  * atanks - obliterate each other with oversize weapons
  * Copyright (C) 2003  Thomas Hudson
@@ -308,3 +311,5 @@ ITEM item[ITEMS] = {
         25000,   1,     FALSE,  2,      0,      {0, 0}}
 };
 
+#endif
+
